#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <pthread.h>

#include "../ipc.h"
#include "../ipc_config.h"

#define RST		printf("\033[0m")
#define RED		printf("\033[0;31m")
#define GREEN	printf("\033[0;32m")
#define BLUE	printf("\033[0;34m")
#define IRED	printf("\033[1;91m")
#define IGREEN	printf("\033[1;92m")
#define IBLUE	printf("\033[1;94m")
#define BRED	printf("\033[0;101m")
#define BGREEN	printf("\033[0;102m")
#define BBLUE	printf("\033[0;104m")

#define NORMAL	IBLUE
#define ERR		BBLUE
#define DATA  BLUE

#define _D(fmt, ...) do {NORMAL; printf(fmt, ##__VA_ARGS__); RST; } while (0)
#define _DATA(fmt, ...) do {DATA; printf(fmt, ##__VA_ARGS__); RST; } while (0)
#define _ERR(fmt, ...) do {ERR; printf(fmt, ##__VA_ARGS__); RST; } while (0)

// refer callback relative functions
extern int callback_send_data(void *data);
extern int callback_init_connection(void);

// server callee parser template
static int IPC_StartServer(char *(*pFunc) (char *, int))
{
	SOCKET_HANDLE handle;
	int ret = 0;
	char *buf = NULL;
	int msgSize = 0;
	char *ret_buf = NULL;

	_D("[SERVER] adaptor_api_server start\n");

	ret = IPC_OpenServerConnection(SERVER, &handle);

	_D("[SERVER] IPC_OpenServerConnection (ret=%d)\n", ret);

	if (ret >= 0) {
		while (1) {
			ret = IPC_ClientAccept(&handle);
			_D("[SERVER] IPC_ClientAccept (ret=%d)\n", ret);

			if (ret != -1) {
				ret = IPC_GetMessage(&handle, &msgSize);
				_D("[SERVER] msgSize = %d\n", msgSize);

				if (msgSize > 0) {
					buf = (char *)malloc(msgSize + 1);
					memset(buf, 0x00, msgSize + 1);

					IPC_RecvMessage(&handle, buf, msgSize);
					_DATA("[SERVER] recvData = %s\n", buf);
					/* Server mainloop */
					ret_buf = (char *)pFunc(buf, strlen(buf));

					_DATA("[SERVER] sendData = %s\n", ret_buf);
					IPC_SendMessage(&handle, ret_buf, strlen(ret_buf));

					if (ret_buf != NULL)
						free(ret_buf);

					if (buf != NULL)
						free(buf);
					_D("[SERVER] ipc completed...<<<<<<<<<<<<<<\n");
				}
			}
		}
		ret = IPC_OpenServerClose(&handle);
	} else {
		_ERR("[SERVER] server open error !!\n");
		return -1;
	}

	_D("[SERVER] close\n");
	return 0;

}

//////////////////////////////////////////////////////////////////
// callee function body
//////////////////////////////////////////////////////////////////
static char *__stub_containers_info(char *data)
{
	_D("[SERVER] __stub_dockzen_get_containers_info\n");
	const char *ret_buf = "{\n" "\"count\": 2,\n" "\"container\": [\n" "{\n" "\"id\": \"iab2xuoil1qux1gsaidv5ty3v\",\n" "\"name\": \"tizen\",\n" "\"image_name\": \"10.113.62.204:443\/headless:v0.1@sha256:77e3292a583ecbb9f043185f8d439d86fb3b0325ae061986f1737991a6623d82\",\n" "\"status\": \"running\"\n" "},\n" "{\n" "\"id\": \"m1emzrg31u6tg2gkzob04s334\",\n" "\"name\": \"agent\",\n" "\"image_name\": \"10.113.62.204:443\/dockzen_agent:v1.0@sha256:32883df052ae8bd7ef3d0786e3e6e4473c113c65973bae9808fb4df42c000241\",\n" "\"status\": \"running\"\n" "}\n" "]\n" "}\n";

	char *p;

	p = malloc(strlen(ret_buf) + 1);
	memcpy(p, ret_buf, strlen(ret_buf));

	return p;
}

//////////////////////////////////////////////////////////////////
// callee function list structure & register
//////////////////////////////////////////////////////////////////
typedef char *(*cmdFunctionPtr) (char *);
struct _cmd_table_s {
	char cmd[50];
	cmdFunctionPtr func;
};
typedef struct _cmd_table_s cmd_mapping_tables;

cmd_mapping_tables cmd_mapping_table[] = {
	{"get_containers_info", __stub_containers_info},
	{"end", NULL}
};

static char *dockzen_api_parser(char *data, int size)
{
	char *func = NULL;
	char *param = NULL;
	char *ret_buf = NULL;
	int i = 0;

	_D("dockzen_api_parser\n");

	func = json_getString(data, "func");
	param = json_getObject(data, "param");

	_D("func:%s\n", func);
	_D("param:%s\n", param);

	while (1) {
		if (!strncmp(cmd_mapping_table[i].cmd, "end", strlen("end")))
			break;
		if (!strncmp(cmd_mapping_table[i].cmd, func, strlen(func))) {
			ret_buf = cmd_mapping_table[i].func(param);
			_D("[SERVER] ret_buf=%p\n", ret_buf);
			break;
		}
		i++;
	}
	_D("[SERVER] ret_buf=%s\n", ret_buf);
	if (func != NULL)
		free(func);
	if (param != NULL)
		free(param);

	_D("[SERVER] call completed...\n");
	return ret_buf;
}

int _server_init(void *pv)
{
	char *(*pFunc) (char *, int) = dockzen_api_parser;

	_D("[SERVER] dockzen_api_init\n");

	if (IPC_StartServer(pFunc) < 0) {
		_ERR("[SERVER] IPC_StartServer Failed\n");
		return -1;
	}
}

//////////////////////////////////////////////////////////////////
// server main
//////////////////////////////////////////////////////////////////

int main(int argc, char **argv)
{
	int status;
	int ret;

	_D("[SERVER] main function...\n");

	ret = callback_init_connection();
	if (0 != ret)
		_ERR("[SERVER] callback_init_connection error (ret=%d)\n", ret);

	ret = _server_init(NULL);

	_D("[SERVER] server closed\n");

	return 1;
}
