/*
 * Copyright (c) 2016-2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#ifndef __SETUP_SYSTEMDATA_H__
#define __SETUP_SYSTEMDATA_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "sa_types.h"
#include "adaptor_api_types.h"

/**
 * @fn        void setup_user_certificates(void)
 * @brief     This function to add user ca-certificates
 */
	void setup_user_certificates(void);

/**
 * @fn        int sa_setup_system(sa_network_s *info, int flagInitialized)
 * @brief     This function to set system info info
   @param     sa_system_s			[in] system info
   @param     int                   [in] flagInitialized
 * @return    sa_error_e            return of function
 */
	sa_error_e sa_setup_system(sa_system_s * system, int flagInitialized);

/**
 * @fn        int sa_device_reboot()
 * @brief     This function to perform system reboot
 */
	int sa_device_reboot(void);

/**
 * @fn        char * sa_create_device_reboot_json_string(int ret_val)
 * @brief     This function to json string of device reboot result
 */
	char *sa_create_device_reboot_json_string(int ret_val);

/**
 * @fn        int sa_get_disk_info(disk_info_s *disk_info_h)
 * @brief     This function to get disk information
 */
	int sa_get_disk_info(disk_info_s * disk_info_h);

/**
 * @fn        char * sa_create_disk_info_json_string(disk_info_s *disk_info_h)
 * @brief     This function to json string of disk information
 */
	char *sa_create_disk_info_json_string(disk_info_s * disk_info_h);

/**
 * @fn        int sa_get_os_info(os_info_s *os_info_h)
 * @brief     This function to get os information
 */
	int sa_get_os_info(os_info_s * os_info_h);

/**
 * @fn        char * sa_createos_info_json_string(os_info_s *os_info_h)
 * @brief     This function to json string of os information
 */
	char *sa_create_os_info_json_string(os_info_s * os_info_h);

/**
 * @fn        int sa_factory_restore()
 * @brief     This function to perform factory restore
 */
	int sa_factory_restore();

/**
 * @fn        char * sa_create_factory_restore_json_string(int ret_val)
 * @brief     This function to json string of factory restore result
 */
	char *sa_create_factory_restore_json_string(int value);

#ifdef __cplusplus
}
#endif
#endif							/* __SETUP_SYSTEMDATA_H__ */
